from .base_controller import BaseController
from models.message import Message
from models.author import Author
from models.base import Session
from config import MESSAGES_PER_PAGE
import logging

logger = logging.getLogger(__name__)

class MessageRetrieval(BaseController):
    @classmethod
    def get_messages(cls, page=1):
        try:
            messages, total_messages, total_chats = Message.get_messages(page, MESSAGES_PER_PAGE)
            
            if not messages and page == 1:
                logger.warning("No messages found in database")
            
            total_pages = (total_messages + MESSAGES_PER_PAGE - 1) // MESSAGES_PER_PAGE
            
            # Calculate page range (show 5 pages around current page)
            start_page = max(1, page - 2)
            end_page = min(total_pages, page + 2)
            page_range = list(range(start_page, end_page + 1))
                
            result = {
                'messages': messages,  # Already converted to dict in get_messages
                'pagination': {
                    'current_page': page,
                    'total_pages': total_pages,
                    'has_next': (page * MESSAGES_PER_PAGE) < total_messages,
                    'total_results': total_messages,
                    'page_range': page_range  # Added page_range
                },
                'stats': {
                    'total_messages': total_messages,
                    'total_chats': total_chats
                }
            }
            
            return result
        
        except Exception as e:
            logger.exception("Error fetching messages")
            return None

    @staticmethod
    def search_messages(query='', search_type='all', page=1):
        try:
            messages, total_messages = Message.search_messages(
                query, search_type, page, MESSAGES_PER_PAGE
            )
            
            total_pages = (total_messages + MESSAGES_PER_PAGE - 1) // MESSAGES_PER_PAGE
            
            return {
                'messages': messages,
                'pagination': {
                    'current_page': page,
                    'total_pages': total_pages,
                    'has_next': page < total_pages,
                    'total_results': total_messages
                }
            }
        except Exception as e:
            logger.error(f"Error searching messages: {str(e)}", exc_info=True)
            return None
        finally:
            Session.remove()

    @staticmethod
    def _get_or_create_author(author_name, author_number):
        """Get or create an author with proper error handling"""
        try:
            return Author.get_or_create(author_name, author_number)
        except Exception as e:
            logger.error(f"Failed to create/get author: {str(e)}")
            try:
                return Author.get_or_create("Unknown Author", None)
            except Exception as inner_e:
                logger.error(f"Failed to create default author: {str(inner_e)}")
                raise

    @classmethod
    def get_messages_by_author(cls, page=1):
        try:
            session = Session()
            # Get total count of messages instead of authors
            total_messages = session.query(Message).count()
            
            result = Message.get_messages_by_author(page, MESSAGES_PER_PAGE)
            if result:
                result['pagination'].update({
                    'total_results': total_messages,
                    'current_page': page,
                    'total_pages': (total_messages + MESSAGES_PER_PAGE - 1) // MESSAGES_PER_PAGE,
                    'has_next': (page * MESSAGES_PER_PAGE) < total_messages
                })
                result['stats'] = {
                    'total_messages': total_messages
                }
            return result
        except Exception as e:
            logger.error(f"Error fetching messages by author: {str(e)}")
            return None
        finally:
            session.close()

    @staticmethod
    def get_duplicate_count():
        """Get the count of duplicate messages processed in the current fetch operation"""
        session = Session()
        try:
            # Count messages where duplicate_count > 0
            duplicate_count = session.query(Message)\
                .filter(Message.duplicate_count > 0)\
                .count()
            return duplicate_count
        except Exception as e:
            logger.error(f"Error getting duplicate count: {str(e)}")
            return 0
        finally:
            session.close()


